!******************************************************************************!
!                                                                              !
!    lcVFaF.f90                                                                !
!    is part of RECOLA2 (REcursive Computation of One Loop Amplitudes)2        !
!                                                                              !
!    Copyright (C) 2016-2020 Ansgar Denner, Jean-Nicolas Lang and              !
!                            Sandro Uccirati                                   !
!                                                                              !
!    RECOLA2 is licenced under the GNU GPL version 3,                          !
!    see COPYING for details.                                                  !
!                                                                              !
!******************************************************************************!
!                                                           .---.              !
!                                                          /  .  \             !
!                                                         |\_/|   |            !
!                                                         |   |  /|            !
!          .----------------------------------------------------' |            !
!         /  .-.   lcVFaF.f90                                     |            !
!        |  /   \  This code was generated with REPT1L.           |            !
!        | |\_.  | Change this file and you get what you deserve. |            !
!        |\|  | /| Author: J.N. Lang                              |            !
!        | `---' | UFO date: 2015                                 |            !
!        |       | UFO version: 0.1                               |            !
!        |       | Generation date: 2022-04-10 11:17             /             !
!        |       |----------------------------------------------'              !
!        \       |                                                             !
!         \     /                                                              !
!          `---'                                                               !
!******************************************************************************!
module lcVFaF
  use class_vertices
  use constants_mdl
  implicit none
  complex (dp) :: x0, x1, x2, x3, x4, x5, x6, x7, x8, x9, x10, x11, x12, x13,  &
                  x14, x15, x16, x17, x18, x19, x20, x21, x22, x23, x24, x25,  &
                  x26, x27, x28, x29, x30, x31, x32, x33, x34, x35, x36, x37,  &
                  x38, x39, x40, x41, x42, x43, x44, x45, x46, x47, x48, x49
  complex (dp), dimension(0:3) :: x0vec, x1vec, x2vec, x3vec, x4vec, x5vec,    &
                                  x6vec, x7vec
  contains
    subroutine loopVFaF(ty, co, wl, P1, PL1, w2, P2, PL2, m, wp, riMaxIn,      &
    riMaxOut, rankInc)
      use class_vertices
      integer, intent(in) :: riMaxIn, riMaxOut, rankInc
      complex (kind = dp), intent(in), dimension(0:3) :: P1, PL1
      complex (kind = dp), intent(in), dimension(0:3, 0:riMaxIn) :: wl
      complex (kind = dp), intent(in), dimension(0:3) :: P2, PL2
      complex (kind = dp), intent(in), dimension(0:3) :: w2
      complex (dp), intent(out), dimension(0:3,0:riMaxOut) :: wp
      complex (kind = dp), intent(in) :: m
      complex (kind = dp), dimension(0:3) :: pl, p
      integer, intent(in) :: ty
      complex (kind = dp), dimension(0:), intent(in) :: co
      complex (kind = dp), dimension(0:3) :: sp1
      complex (kind = dp), dimension(0:3,0:3) :: sp2
      complex (kind = dp), dimension(0:3,0:3,0:3) :: sp3
      complex (kind = dp), dimension(0:3,0:3,0:3,0:3) :: sp4
      character(len=3) :: ty_str
      integer :: riOut, j
      logical :: isFirstRI
      integer :: mu, nu, ro, ta, te, tu
      select case(ty)
      case(VFaF0)
!       GammaP(1,2,3)*c0
!       GammaM(1,2,3)*c1
        pl(:) = PL1(:) + PL2(:)
        x3 = cima*pl(1)
        x5 = co(1)*w2(0)
        x8 = cima*pl(2)
        x11 = co(1)*w2(1)
        x14 = co(0)*m
        x20 = cima*pl(3)
        x23 = cima*pl(0)
        x32 = co(0)*w2(2)
        x33 = co(0)*w2(3)
        x35 = co(1)*m
        do j = riMaxIn, 0, -1
          x0 = pl(2)*wl(2,j)
          x1 = cima*wl(1,j)
          x2 = pl(2)*x1
          x4 = wl(0,j)*x3 + wl(3,j)*x3
          x6 = pl(1)*wl(2,j)
          x7 = pl(1)*x1
          x9 = wl(3,j)*x8
          x10 = wl(0,j)*x8
          x12 = -wl(2,j) + x1
          x13 = w2(3)*x12
          x15 = wl(0,j) - wl(3,j)
          x16 = w2(2)*x15
          x17 = cima*x16
          x18 = pl(0)*wl(2,j)
          x19 = pl(0)*x1
          x21 = -wl(0,j)*x20
          x22 = wl(3,j)*x20
          x24 = wl(0,j)*x23
          x25 = wl(3,j)*x23
          x26 = pl(3)*wl(2,j) - pl(3)*x1
          x27 = wl(2,j) + x1
          x28 = w2(2)*x27
          x29 = wl(0,j) + wl(3,j)
          x30 = w2(3)*x29
          x31 = cima*x30
          x34 = w2(1)*x12
          x36 = w2(0)*x29
          x37 = cima*x36
          x38 = w2(0)*x27
          x39 = w2(1)*x15
          x40 = cima*x39
          x41 = -co(1)*(x34 + x37)
          x42 = co(1)*(x38 + x40)
          x43 = -x42
          x44 = co(0)*(-x13 + x17)
          x45 = co(0)*(-x28 + x31)
          x46 = -x45
          x47 = cima*wl(2,j)
          x48 = -wl(1,j) + x47
          x49 = wl(1,j) + x47
          wp(0,j) = -x11*(-x10 - x6 + x7 + x9) + x13*x14 - x14*x17 - x5*(-x0 - &
                    x2 + x4)
          wp(1,j) = -x11*(x24 - x25 + x26) + x14*x28 - x14*x31 - x5*(x18 + x19 &
                     + x21 - x22)
          wp(2,j) = x32*(x0 + x2 - x24 + x25) - x33*(x10 + x18 - x19 + x9) -   &
                    x34*x35 - x35*x37
          wp(3,j) = x32*(x21 + x22 + x6 + x7) - x33*(x26 + x4) - x35*x38 - x35*&
                    x40
          if(rankInc .lt. 1) cycle
          sp2(0,0) = x41
          sp2(0,1) = x43
          sp2(0,2) = -x44
          sp2(0,3) = x46
          sp2(1,0) = x43
          sp2(1,1) = x41
          sp2(1,2) = x45
          sp2(1,3) = x44
          sp2(2,0) = -co(1)*(w2(0)*x48 - x39)
          sp2(2,1) = -co(1)*(w2(1)*x49 + x36)
          sp2(2,2) = -co(0)*(w2(2)*x48 + x30)
          sp2(2,3) = co(0)*(-w2(3)*x49 + x16)
          sp2(3,0) = x41
          sp2(3,1) = x42
          sp2(3,2) = x44
          sp2(3,3) = x46
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(:,riOut) = wp(:,riOut) + sp2(mu,:)
          end do 
        end do 
      case(VFaF1)
!       GammaM(1,2,3)*c0
        pl(:) = PL1(:) + PL2(:)
        x0 = cima*pl(1)
        x2 = cima*pl(2)
        x3 = cima*pl(3)
        x4 = cima*pl(0)
        do j = riMaxIn, 0, -1
          x1 = cima*wl(1,j)
          x5 = w2(0)*(wl(0,j) + wl(3,j))
          x6 = co(0)*(cima*x5 + w2(1)*(-wl(2,j) + x1))
          x7 = w2(1)*(wl(0,j) - wl(3,j))
          x8 = co(0)*(cima*x7 + w2(0)*(wl(2,j) + x1))
          x9 = -x6
          x10 = -x8
          x11 = cima*wl(2,j)
          wp(0,j) = -co(0)*(w2(0)*(-pl(2)*wl(2,j) - pl(2)*x1 + wl(0,j)*x0 + wl(&
                    3,j)*x0) + w2(1)*(-pl(1)*wl(2,j) + pl(1)*x1 - wl(0,j)*x2 + &
                    wl(3,j)*x2))
          wp(1,j) = -co(0)*(w2(0)*(pl(0)*wl(2,j) + pl(0)*x1 - wl(0,j)*x3 - wl(3&
                    ,j)*x3) + w2(1)*(pl(3)*wl(2,j) - pl(3)*x1 + wl(0,j)*x4 - wl&
                    (3,j)*x4))
          wp(2,j) = -m*x6
          wp(3,j) = -m*x8
          if(rankInc .lt. 1) cycle
          sp2(0,0) = x9
          sp2(0,1) = x10
          sp2(0,2) = 0
          sp2(0,3) = 0
          sp2(1,0) = x10
          sp2(1,1) = x9
          sp2(1,2) = 0
          sp2(1,3) = 0
          sp2(2,0) = -co(0)*(w2(0)*(-wl(1,j) + x11) - x7)
          sp2(2,1) = -co(0)*(w2(1)*(wl(1,j) + x11) + x5)
          sp2(2,2) = 0
          sp2(2,3) = 0
          sp2(3,0) = x9
          sp2(3,1) = x8
          sp2(3,2) = 0
          sp2(3,3) = 0
          do mu = 0, 3
            riOut = incRI(mu, j)
            wp(:,riOut) = wp(:,riOut) + sp2(mu,:)
          end do 
        end do 
      end select 
    end subroutine loopVFaF
end module lcVFaF
